#!/system/bin/sh
MODDIR=${0%/*}

# Paths
BASEDIR=/data/adb/modules/APreload
INT=/storage/emulated/0
RWD=$INT/APreload
MSC=$BASEDIR/preload

notification() {
    su -lp 2000 -c "cmd notification post -S bigtext -t '$1' tag '$2'" >/dev/null 2>&1
}

# Ensure the rewrite directory exists
if [ ! -d "$RWD" ]; then
  mkdir -p "$RWD"
fi

# Check applist file
if [ ! -e "$RWD/applist_perf.txt" ]; then
  cp -f "$MSC/applist_perf.txt" "$RWD"
fi

# Function to get total RAM in GB
total_ram() {
    MEM_TOTAL=$(grep MemTotal /proc/meminfo | awk '{print $2}')
    echo "scale=2; ($MEM_TOTAL + 1048576) / 1048576" | bc
}

# Run one-time OpenGL/Vulkan library handling
for ssl in $(find /*/lib -type f); do
    if strings "$ssl" | grep -Eio 'opengl|vulkan'; then
        vmt -dL "$ssl"
    fi
done &
sed -Ei 's/^description=(\[.*][[:space:]]*)?/description=[ ♻️ Checking ] /g' "$BASEDIR/module.prop"
notification "Preload Auto" "Waiting for 1 Minutes"
# Delay for 60 seconds
sleep 60
# Kill vmt process
pkill -f vmt
notification "Preload Auto" "Preload Checking Succesfull"

# Start AI loop
while true; do
    sleep 15
    
    # Build app filter list
    app_list_filter="grep -o -e applist.app.add"
    while IFS= read -r applist || [[ -n "$applist" ]]; do
        filter=$(echo "$applist" | awk '!/ /')
        if [[ -n "$filter" ]]; then
            app_list_filter+=" -e $filter"
        fi
    done < "$RWD/applist_perf.txt"
    
    # Check if an app from the applist is active
    window=$(dumpsys window | grep package | $app_list_filter | tail -1) 
    if [[ -n "$window" ]]; then
        package=$(echo "$window" | awk '{print $NF}')
        pkill -f vmt
        # Get total RAM
        MEM_GB=$(total_ram)
        if (( $(echo "$MEM_GB < 5" | bc -l) )); then
            vmt -dL "$(cmd package path android | cut -f2 -d:)"
            vmt -dL "$(cmd package path com.android.systemui | cut -f2 -d:)"
        sed -Ei 's/^description=(\[.*][[:space:]]*)?/description=[ ⚡ RUNNING ] /g' "$BASEDIR/module.prop"
        notification "Preload Auto" "Preload Running..."
        sleep 60
        else
            for sl in $(find /*/lib -type f); do
                if strings "$sl" | grep -Eio 'opengl|vulkan'; then
                    vmt -dL "$sl"
                fi
            done &
            for al in $(find "$(cmd package path "$window"|head -n1|cut -f2 -d:|xargs -n1 dirname)/lib" -type f);do
                if strings "$al"|grep -Eio 'opengl|vulkan';then
                    vmt -dL "$al"
                fi
            done &
            for tg in $(cmd package path "$window"|grep -Ei 'arm|base|x86'|cut -f2 -d:) /storage/emulated/0/Android/data/$window/cache;do
                vmt -dL "$tg"
            done&
        fi

        # Update module description to Running
        sed -Ei 's/^description=(\[.*][[:space:]]*)?/description=[ ⚡ RUNNING ] /g' "$BASEDIR/module.prop"
        sleep 60
    else
        pkill -f vmt
        sed -Ei 's/^description=(\[.*][[:space:]]*)?/description=[ ⛔ STOPPED ] /g' "$BASEDIR/module.prop"
        sleep 60
    fi
done &
